package Wrangler::License;
#
# Warning: This module was auto-generated by build_textmodules.pl, manual changes will be lost!
#

use strict;
use warnings;

our $text = "--------------------------------------------------------------------------------

Despite Wrangler being released as open-source software, it is not \"open source\"
as per official definition, nor is it \"free software\", as it is licensed to
end-users under two restrictive licenses, distinguishing between non-commercial
and for-profit/ commercial usage contexts.

--------------------------------------------------------------------------------

  For you to compare, the following is a summary of the licenses
  (but not a substitute for the legal text):

  a) Wrangler Non-Commercial License

     For non-commercial, personal, not-for-profit use.

  b) Wrangler Commercial License

     For use of Wrangler in commercial contexts, for example at work, or for usage
     scenarios where you use Wrangler as part of a for-profit activity.

  Both licenses are essentially the same, with the sole difference being the allowed
  use context, private vs. commercial.

  Both licenses are giving you permission to distributed Wrangler, provided the
  distribution package is not modified and provided that the distribution itself
  is not for-profit.

  Both licenses grant permission to modify the source-code, but you may not publish
  these modified versions without restrictions. If you decide to do so, for example
  by \"forking\" and then \"pushing\" your changes to a public source code repository,
  like github, you warrant that you are the original author and you agree to waive
  your rights to your contribution.

  Both licenses require that Wrangler may not be sold individually, or be bundled
  with other commercial products. Other licensing terms are available by mutual
  agreement, for example if you want to distribute Wrangler unmodified but for profit,
  or want to distribute/ circulate/ sell Wrangler in a modified form.

--------------------------------------------------------------------------------


Wrangler Non-Commercial License 1.0

This SOFTWARE PRODUCT (Wrangler) is protected by copyright laws and international
copyright treaties, as well as other intellectual property laws and treaties. The
SOFTWARE PRODUCT is licensed, not sold.

This licence is a legal agreement between you (the \"user\", the \"licensee\") and Clipland
GmbH (\"the author\"), and becomes a binding contract between you and Clipland GmbH.
This agreement governs the terms of use of Wrangler, together with any media, printed
materials, electronic documentation, updates, add-ons, artwork, web services and
any other material that may be associated with the product now or in the future.
This agreement becomes effective (a) when you \"accept licence agreement,\" or (b)
if you acquire and accept Wrangler on a storage media, or (c) when you open the
compressed electronic file in which Wrangler is contained. If you do not wish to
enter into this agreement, do not purchase, access, download and/or install or
otherwise use Wrangler.


Usage:

Wrangler's licensing distinguishes between \"use contexts\". Using Wrangler exclusively
in a private, not-for-profit endeavour, or as part of an evaluation process not
longer than 1 day, is regarded as \"private\" use and is permitted free of charge
under the Wrangler Non-Commercial License.
If Wrangler is used directly or indirectly in an institutional, educational or business
environment, as part of a for-profit activity, or in a commercial context, then
this is regarded as \"commercial\" use for which the author requires you to purchase
a Wrangler license key, resulting in Clipland GmbH issuing a Wrangler Commercial
License.
If you are unsure whether your use is not permitted, contact Clipland GmbH. Your
failure to contact to seek permission or the lack of a specific prohibition in this
agreement shall not be interpreted or deemed a waiver or permissible use of any
kind.


  NON-COMMERCIAL LICENSE
  By this agreement, Clipland GmbH grants you a non-exclusive terminable license
  to compile and execute Wrangler free-of-charge for non-commercial, private use;
  provided that the legal notices are maintained intact and honoured.


Licenses are per user and valid for use on all supported operating systems. License
keys may be used on multiple computers and operating systems, provided the license
key holder is the primary user AND provided the use context is mutually exclusive
either \"private\" or \"commercial\". As such, businesses (\"commercial\" use context)
must purchase at least as many licenses as the number of people using Wrangler.


Redistribution:

Redistribution and copying of this release is permitted, provided that:

This package remains unchanged, in its original form as released by the author.
Source code, product and package naming, copright notices and this license may not
be changed. Although this license permits local creation of modifications, it does
not permit the public circulation of derivate works.

This package may not be sold or released under any paid license. Copying fees for
the transport of this package may be imposed, as long as they are part of of a general
non-profit concern. For example, ad placement to fund hosting of the software, or
charging small fees to finance physical media like CD-ROMs is allowed.

Any other distribution of Wrangler, especially modified derivates, requires express
written consent from Clipland GmbH.


Patching and derivate works (modifications):

You may modify Wrangler's source-code for non-public, non-redistributed internal
use. Clipland GmbH encourages users to modify Wrangler and submit these patches,
bugfixes or improvements to the author for possible inclusion in future Wrangler
releases, with the understanding that they would be treated the same as all other
Wrangler code.

You may submit patches via public means, for example by \"forking\" official Wrangler
code repositories, and then \"pushing\" your changes to your public source code repository.
However, a user \"forking\" code repositories understands that this type of submission
is solely intended for \"patching\", with changes being minor. A \"fork\" has to be
temporary, and does not constitute a modified distribution as a whole, and must
be removed by written request of Clipland GmbH. The user will not promote any unofficial
repositories and their modifications until they have been evaluated and approved
or have become part of the official release.

Prior to the submission of patches by any means, the author requires you to identify
yourself as the sole originator of the code, and that you warrant your patch is
not built upon or derived from foreign works and the submission does not infringe
third party rights. With the submission you further grant an unlimited, perpetual
license free of charge to Clipland GmbH for evaluation and and inclusion of your
modifications, in case they become part of official Wrangler releases.


Et cetera:

You may make copies of the license key and or Wrangler for backup and archival purposes.
You must not remove or alter any copyright notices on any copy of Wrangler.
Wrangler or Wrangler license keys may not be sold individually or bundled with other
commercial products. License keys will be revoked if distributed or sold to third parties.
You may not rent, lease, or lend Wrangler license keys or Wrangler.
All title, including but not limited to copyrights, in and to Wrangler and any copies
thereof are owned by Clipland GmbH.

Clipland Wrangler IS LICENSED TO YOU \"AS IS,\" AND WITHOUT ANY WARRANTY OF ANY KIND,
WHETHER ORAL, WRITTEN, EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED
TO WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. 
THE AUTHOR DOES NOT WARRANT THAT THIS SOFTWARE DOES NOT INFRINGE ANY RIGHTS OF THIRD
PARTIES.

This agreement will terminate automatically if you fail to comply with the limitations
described herein. Upon termination, you must destroy all copies of this package,
the software, and any derivatives within 48 hours.

In no event shall Clipland GmbH be liable for any damages due to use or misuse of
Wrangler, to the maximum extent permitted by law. This includes without limitation,
lost profits, business interruption, or lost information. In no event will Clipland
GmbH be liable for loss of data or for indirect, special, incidental, consequential
 - including lost profit, or other damages. Clipland GmbH shall have no liability
with respect to the content of Wrangler or any part thereof, including but not limited
to errors or omissions contained therein, libel, trademark rights, business interruption,
loss of privacy or the disclosure of confidential information.

Other licensing terms are available by mutual agreement.


This license document itself, the \"Wrangler License\", is copyright 2014 Clipland
GmbH and licensed under the GNU Free Documentation License (GFDL/FDL).


--------------------------------------------------------------------------------


Wrangler Commercial License 1.0

This SOFTWARE PRODUCT (Wrangler) is protected by copyright laws and international
copyright treaties, as well as other intellectual property laws and treaties. The
SOFTWARE PRODUCT is licensed, not sold.

This licence is a legal agreement between you (the \"user\", the \"licensee\") and Clipland
GmbH (\"the author\"), and becomes a binding contract between you and Clipland GmbH.
This agreement governs the terms of use of Wrangler, together with any media, printed
materials, electronic documentation, updates, add-ons, artwork, web services and
any other material that may be associated with the product now or in the future.
This agreement becomes effective (a) when you \"accept licence agreement,\" or (b)
if you acquire and accept Wrangler on a storage media, or (c) when you open the
compressed electronic file in which Wrangler is contained. If you do not wish to
enter into this agreement, do not purchase, access, download and/or install or
otherwise use Wrangler.


Usage:

Wrangler's licensing distinguishes between \"use contexts\". Using Wrangler exclusively
in a private, not-for-profit endeavour, or as part of an evaluation process not
longer than 1 day, is regarded as \"private\" use and is permitted free of charge
under the Wrangler Non-Commercial License.
If Wrangler is used directly or indirectly in an institutional, educational or business
environment, as part of a for-profit activity, or in a commercial context, then
this is regarded as \"commercial\" use for which the author requires you to purchase
a Wrangler license key, resulting in Clipland GmbH issuing a Wrangler Commercial
License.
If you are unsure whether your use is not permitted, contact Clipland GmbH. Your
failure to contact to seek permission or the lack of a specific prohibition in this
agreement shall not be interpreted or deemed a waiver or permissible use of any
kind.


  COMMERCIAL LICENSE
  Upon purchase of a license key, Clipland GmbH will grant you this non-exclusive
  terminable license to use Wrangler. This commercial license permits the licensee
  to compile, execute, use Wrangler for any purpose, including commercial; provided
  that the legal notices are maintained intact and honoured.


Licenses are per user and valid for use on all supported operating systems. License
keys may be used on multiple computers and operating systems, provided the license
key holder is the primary user AND provided the use context is mutually exclusive
either \"private\" or \"commercial\". As such, businesses (\"commercial\" use context)
must purchase at least as many licenses as the number of people using Wrangler.


Redistribution:

Redistribution and copying of this release is permitted, provided that:

This package remains unchanged, in its original form as released by the author.
Source code, product and package naming, copright notices and this license may not
be changed. Although this license permits local creation of modifications, it does
not permit the public circulation of derivate works.

This package may not be sold or released under any paid license. Copying fees for
the transport of this package may be imposed, as long as they are part of of a general
non-profit concern. For example, ad placement to fund hosting of the software, or
charging small fees to finance physical media like CD-ROMs is allowed.

Any other distribution of Wrangler, especially modified derivates, requires express
written consent from Clipland GmbH.


Patching and derivate works (modifications):

You may modify Wrangler's source-code for non-public, non-redistributed internal
use. Clipland GmbH encourages users to modify Wrangler and submit these patches,
bugfixes or improvements to the author for possible inclusion in future Wrangler
releases, with the understanding that they would be treated the same as all other
Wrangler code.

You may submit patches via public means, for example by \"forking\" official Wrangler
code repositories, and then \"pushing\" your changes to your public source code repository.
However, a user \"forking\" code repositories understands that this type of submission
is solely intended for \"patching\", with changes being minor. A \"fork\" has to be
temporary, and does not constitute a modified distribution as a whole, and must
be removed by written request of Clipland GmbH. The user will not promote any unofficial
repositories and their modifications until they have been evaluated and approved
or have become part of the official release.

Prior to the submission of patches by any means, the author requires you to identify
yourself as the sole originator of the code, and that you warrant your patch is
not built upon or derived from foreign works and the submission does not infringe
third party rights. With the submission you further grant an unlimited, perpetual
license free of charge to Clipland GmbH for evaluation and and inclusion of your
modifications, in case they become part of official Wrangler releases.


Et cetera:

You may make copies of the license key and or Wrangler for backup and archival purposes.
You must not remove or alter any copyright notices on any copy of Wrangler.
Wrangler or Wrangler license keys may not be sold individually or bundled with other
commercial products. License keys will be revoked if distributed or sold to third parties.
You may not rent, lease, or lend Wrangler license keys or Wrangler.
All title, including but not limited to copyrights, in and to Wrangler and any copies
thereof are owned by Clipland GmbH.

Clipland Wrangler IS LICENSED TO YOU \"AS IS,\" AND WITHOUT ANY WARRANTY OF ANY KIND,
WHETHER ORAL, WRITTEN, EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED
TO WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. 
THE AUTHOR DOES NOT WARRANT THAT THIS SOFTWARE DOES NOT INFRINGE ANY RIGHTS OF THIRD
PARTIES.

This agreement will terminate automatically if you fail to comply with the limitations
described herein. Upon termination, you must destroy all copies of this package,
the software, and any derivatives within 48 hours.

In no event shall Clipland GmbH be liable for any damages due to use or misuse of
Wrangler, to the maximum extent permitted by law. This includes without limitation,
lost profits, business interruption, or lost information. In no event will Clipland
GmbH be liable for loss of data or for indirect, special, incidental, consequential
 - including lost profit, or other damages. Clipland GmbH shall have no liability
with respect to the content of Wrangler or any part thereof, including but not limited
to errors or omissions contained therein, libel, trademark rights, business interruption,
loss of privacy or the disclosure of confidential information.

Other licensing terms are available by mutual agreement.


This license document itself, the \"Wrangler License\", is copyright 2014 Clipland
GmbH and licensed under the GNU Free Documentation License (GFDL/FDL).
";

1;