package Wrangler::Changes;
#
# Warning: This module was auto-generated by build_textmodules.pl, manual changes will be lost!
#

use strict;
use warnings;

our $text = "Revision history for Wrangler

2.11	upcoming (first public release)
	- CustomizableEditor is back, on the 2.x branch
	  as FormEditor, with many improvements, switchable
	  layouts, UI for adding/removing fields, etc.
	- MultiRename dialog ported to 2.x branch
	- Previewer displays video thumbnails again
	- FileBrowser: configurable columns complete,
	  sorting remembered
	- ColourLabels Plugin
	- Layers provides new field \"Filesystem::Xattr\"
	- Value Shortcuts facility ported to 2.x branch
	- Settings UI complete and refined
	- accepts (--debug 1) for debug output to STDOUT
	- silenced/configurable debug output
	- reworked Plugin API (still WIP)
	- stability fixes
	- app-icon-set completed, optional scalable
	- polished POD, with screenshots

2.10	2014-01-18 (unreleased)
	- MetadataThumb is back, on the 2.x branch as
	  Previewer, with fixes
	- preparation for dynamic layouting
	- Navbar with history and clickable path
	- Sidebar with icons
	- configurable, MIME-Type based viewers facility
	- new Settings UI, more config bindings 
	- Debian .deb file pipeline working
	- new Images.pm build process
	- License and Changes embedded as modules

2.07	2013-12-15 (unreleased)
	- initial PubSub
	- basic Sidebar
	- OO-interface dropped on selected subs for speed
	- ListCtrl FileBrowser mainly done, restored
	  configurable columns, zebra-striping, highlighting
	- new logo, new internal image/icon handling
	- ported tests

2.02	2013-10-30 (unreleased)
	- Wrangler::Config rewrite
	- implementation of FileBrowser as a TreeList halted
	- configurable widget layout
	- new PluginManager

2.0	2013-07-01 (unreleased)
	- start of new major version, branch/rewrite
 	- switch from YAML to JSON for settings
	- new XDG config file location
	- removed sidecar-file handling for now
	- many widgets and dialogs not yet ported to 2.x
	- new underlying file i/o
	- removed separate handling via Metadata.pm
	- FileSystem::Layers
	- native file/folder/icons
	- new, WIP Plugin API
	- events replace hardcoded bindings
	- new module layout
	- better separation of WxWidgets and Wrangler's
	  internal data structures
	- CPAN bundling and proper Licensing

1.36	2013-04-28 (unreleased)
	- fixed video preview and thumbnailing
	- optimized onpaint/redraws
	- stability: various crashers fixed

1.24	2012-02-07 (unreleased)
	- Layered VFS and 'mounting' services
	- Dialog::AdjustTimestamp
	- switchable Metadata editors: CustomizableEditor
	  and MetadataPanel/MetadataTree
	- internal metadata API via Metadata.pm

1.17	2011-08-16 (unreleased)
	- Dialog::PlaylistEditor (ASX, XML, M3U, ..)
	- FileBrowser zooming/fontsize
	- export listings as text
	- initial metadata mapping tool OneToMany
	- more filetypes, WebM etc.
	- alternative metadata editor CustomizableEditor
	- Previewer with slider: pos and time
	- internal/native \"viewers\" for Images, Video
	  and Audio
	- insert editable text-templates via configurable
	  keyboard shortcuts
	- PAR packaging
	- cross platform fixes for Windows and Linux

1.0	2010-11-01 (unreleased)
	- major rewrites in many areas
	- FileBrowser is not a WxGrid anymore
	- FileBrowser as HTML window prototype
	- MetadataPanel display and editor
	- MetadataThumb previewer
	- new column sorting
	- fixed utf8 (filename) handling
	- proper Settings UI
	- sidecar-file handling and in-listing indenting
	- Accounts manager
	- Dialog::MultiRename
	- i18n

0.5	2009-02-15 (unreleased)
	- dropped tabbed main interface
	- metadata handling logic changed
	- switch to sqlite for backend db
	- l18n internationalization ready
	- unified internal module and data structure
	- file-browsing rewritten
	- working trancoding wizard, Convert.pm
	- renaming from 'Video Wrangler' to 'Wrangler'
	  reflecting an universal approach to media asset
	  handling
	- countless bugfixes, changes
	- Db backend dropped, pure xattr from now on
	- Metadata Details display

0.4	2009-02-19 (unreleased)
	- Dialog::Watermark
	- Plugin::Veoh, Plugin::MySpace

0.3	2008-08-13 (unreleased)
	- Plugin::Dailymotion
	- Dialog::Export

0.2	2008-07-04 (unreleased)
	- switch to WxWidgets
	- early Plugin interface
	- initial upload facility

0.1	2008-03-11 (unreleased)
	- initial Tk version
	- interface design prototype
	- Storable-based backend db
";

1;